; MODULE COM_MAC
;-------------------------------------------------------------------------
; Some useful macroses, not implemented in MPASM
; and the list of internal MPASM' macroses.
; (c)1995,96 Alex Torres 2:461/28@FidoNet aka altor@cook.kharkov.ua
;-------------------------------------------------------------------------
                            LIST
;               p
                            NOLIST
                            ERRORLEVEL -302
; EI       - Enable  Interrupt
; DI       - Disable Interrupt
; PAGESET  - RAM bank switching
; PPAGESET - ROM bank switching (below 4k !!!)
; mov  R1,R2  -  [R2]->[R1]
; movl R,N    -   N->[R]
; cp   R,N    -   compare [R] with N, set Z & W=0 if equal
;                 if [R]<N C=0 Z=0; if [R]>N C=1,Z=0
; cpr  R1,R2  - compare [R1] c [R2], [R1]=[R2] C=1,Z=1
;                                    [R1]<[R2] C=0,Z=0
;                                    [R1]>[R2] C=1,Z=0  , C=1 if [R1]>=[R2]
; cpw & cpwr  - same for 16 bit, low-high
; djnz  reg,addr - Ohh my lovely Z80 !
; comfw       - W= NOT W
; jmp  addr - same as "goto addr"
; xcall & xgoto - cross page (below 4kb !!!)
;-------------------------------------------------------------------------
djnz         macro reg,addr
                decfsz reg,f
                goto   addr
             endm
;---------------------------------------------------
comfw        macro
                xorlw 0xFF
             endm
;---------------------------------------------------
jmp          macro addr
                goto addr
             endm
;---------------------------------------------------
cp           macro reg,number
                movlw number
                subwf reg,w
             endm
;---------------------------------------------------
cpr          macro reg1,reg2
                movf reg2,w
                subwf reg1,w
             endm
;---------------------------------------------------
cpw          macro reg,number
             local cpw_
                movf reg,w
                addlw low(-number)
                movf reg+1,w
                skpnc
                addlw 1
                addlw high (-number)
cpw_
             endm
;---------------------------------------------------
cpwr         macro reg1,reg2
              local cpwr_
                cpr reg1+1, reg2+1
                skpc
                goto cpwr_
                skpz
                goto cpwr_
                cpr reg1, reg2
cpwr_
             endm
;---------------------------------------------------
mov          macro out,in
                movf  in,w
                movwf out
             endm
movl         macro  out,in
                movlw in
                movwf out
             endm
;------------------------------------------
;            RAM bank switching
PAGESET      macro  number;
               if (number==1)
                  bsf  STATUS,RP0
               else
                  bcf  STATUS,RP0
               endif
             endm
;------------------------------------------
;          ROM bank switching
PPAGESET     macro   number
               if (number==1)
                  bsf  PCLATH,3
               else
                  bcf  PCLATH,3
               endif
             endm
;-------------------------------------------
EI           macro       ; Interrupt Enable
                  bsf INTCON,GIE  ;
             endm
;-------------------------------------------
DI           macro       ; Interrupt Disable
              local             di_loop
di_loop           bcf   INTCON,GIE  ;
                  btfsc INTCON,GIE
                  goto  di_loop
             endm
;------------------------------------------------------------
; push/pop W & STATUS  during interrupt
; define SSave & WSave registers !!!
SAVEREG      macro
                  movwf      WSave       ; save W & Status
                  swapf      STATUS,W
                  movwf      SSave
             endm
;---------------------------------------------
RESTREG      macro
                  swapf      SSave,W     ; restore W & Status
                  movwf      STATUS
                  swapf      WSave,F
                  swapf      WSave,W
             endm
;-----------------------------------------------
xcall        macro   routine         ;set ROM page bit
               if  ((routine) & 0x800)
                  bsf  PCLATH,3
               else
                  bcf  PCLATH,3
               endif
               if   ((routine) & 0x800)==($ & 0x800)
                  messg Should use 'call' instead of 'xcall' for routine
               endif
               errorlevel  -306
                  call  routine
               errorlevel  +306
               if ($ & 0x800)
                  bsf PCLATH,3
               else
                  bcf PCLATH,3
               endif
             endm
;-------------------------------
xgoto        macro  routine         ;flip ROM page bit
              if ((routine) & 0x800)
                  bsf PCLATH,3
              else
                  bcf PCLATH,3
              endif
              if ((routine) & 0x800)==($ & 0x800)
                  messg     Should use 'goto' instead of 'xgoto' for routine
              endif
              errorlevel  -306
                  goto  routine
              errorlevel  +306
             endm
;==============================================
; Internal MPASM's MACRO :
;
; lcall addr = bcf/bsf PCLATH,3   Call across page boundary
;              bcf/bsf PCLATH,4
;              call addr
;
; clrc   = bcf STATUS,C             Clear Carry
; clrdc  = bcf STATUS,DC            Clear Digit Carry
; clrz   = bcf STATUS,Z             Clear Zero
; setz   = bsf STATUS,Z             Set Carry
;
; skpc   = btfss STATUS,C           Skip on Carry
; skpnc  = btfsc STATUS,C           Skip on No Carry
; skpdc  = btfss STATUS,DC          Skip on Digit Carry
; skpndc = btfsc STATUS,DC          Skip on No Digit Carry
; skpz   = btfss STATUS,Z           Skip on Zero
; skpnz  = btfsc STATUS,Z           Skip on No Zero
;
; tstf  reg   = movf reg,f          Test File
; movfw reg   = movf reg,w          Move File to W
;
; negf  reg,d = comf reg,f          Negate File
;               incf reg,d
;
; addcf  reg,d =  btfsc STATUS,C    Add Carry to File
;                 incf  reg,d
; subcf  reg,d =  btfsc STATUS,C    Sub Carry from File
;                 decf  reg,d
;
; adddcf reg,d =  btfsc STATUS,DC   Add Digit Carry to File
;                 incf  reg,d
; subdcf reg,d =  btfsc STATUS,DC   Sub Digit Carry to File
;                 decf  reg,d
;
; B   addr  = goto addr             Branch
; BC  addr  = skpnc                 Branch on Carry
;             goto addr
; BNC addr  = skpc                  Branch on No Carry
;             goto addr
; BDC addr  = skpndc                Branch on Digit Carry
;             goto addr
; BNDC addr = skpdc                 Branch on No Digit Carry
;             goto addr
; BZ  addr  = skpnz                Branch on Zero
;             goto addr
; BNZ addr =  skpz                 Branch on No Zero
;             goto addr
;--------------------------------------------------------------------------